/* ---------------------------------------------------------------------------
Programme : OPERIMAGE.C
Langage   : C sur UNIX
Date      : Janvier 1993

Auteur    : Jean-Francois Guilbert   GUIJ05016601
Modifie   : Francois Meunier 

   Ce module comprend les operations relatives a l'image.
--------------------------------------------------------------------------- */

#include "image.h"

/* ---------------------------------------------------------------------------
Fonction OUVRIR_2_IMAGE_RAST : Cette fonction ouvre les fichiers necessaire au
                       traitement d'une image.  Les descripteurs et noms des
fichiers sont conserves dans la strcuture 'Image', dont le pointeur est
retourne a la fonction appelante.

appel : image = ouvrir_2_Image_RAST (argv, argc)
--------------------------------------------------------------------------- */
Image * ouvrir_2_Image_RAST (char *arguments[])
{
  Image *image;                 /* Nouvelle image retournee par la fonction */

  if ((image = (Image *) malloc (sizeof (Image))) == (Image *) NULL)
  {
     printf ("\nInfo : Memoire non disponible pour traitement (ouvirImage)...");
     exit(1);

  } /* if then */

  if ((image->fichiers.fde = fopen (arguments[1], "rb")) == (FILE *) NULL)
  {
     printf ("\nInfo : Fichier source '%s' inexistant.\n", arguments[1]);
     exit (1);

  } /* if then */

  if ((image->fichiers.fds = fopen (arguments[2], "wb")) == (FILE *) NULL )
  {
     printf ("\nInfo : Ne peut creer fichier destination '%s'.\n",
                       arguments[2]);
     exit (2);

  } /* if then */

  image->fichiers.fichierS = arguments[1];
  image->fichiers.fichierD = arguments[2];

  return image;

} /* ouvrirImage */

/* ---------------------------------------------------------------------------
Fonction OUVRIR_1_IMAGE_RAST_et_1_RAW : Cette fonction ouvre les fichiers   necessaire au traitement d'une image.  Les descripteurs et noms des
fichiers sont conserves dans la structure 'Image', dont le pointeur est
retourne a la fonction appelante. Dans ce cas il y a un fichier image
 en entree et un de type RAW en sortie.

appel : image = ouvrir_1_Image_RAST_et_1_RAW (argv, argc)
--------------------------------------------------------------------------- */
Image * ouvrir_1_Image_RAST_et_1_RAW (char *arguments[])
{
  Image *image;                 /* Nouvelle image retournee par la fonction */

  if ((image = (Image *) malloc (sizeof (Image))) == (Image *) NULL)
  {
     printf ("\nInfo : Memoire non disponible pour traitement (ouvirImage)...");
     exit(1);

  } /* if then */

  if ((image->fichiers.fde = fopen (arguments[1], "rb")) == (FILE *) NULL)
  {
     printf ("\nInfo : Fichier source '%s' inexistant.\n", arguments[1]);
     exit (1);

  } /* if then */

  if ((image->fichiers.fds = fopen (arguments[2], "wb")) == (FILE *) NULL )
  {
     printf ("\nInfo : Ne peut creer fichier destination '%s'.\n",
                       arguments[2]);
     exit (2);

  } /* if then */

  image->fichiers.fichierS = arguments[1];
  image->fichiers.fichierD = arguments[2];
  return image;

} /* ouvrirImage */

/* ---------------------------------------------------------------------------
Fonction LIREIMAGE : Cette fonction effectue la lecture des informations
                     relatives a une image (tete, table des couleurs et les
pixels de l'image).  Le parametre 'info' permet l'affichage ou non de
l'information contenue dans la tete du fichier.

Valeurs possible du parametre 'info' :

        0 : n`affiche pas les informations de la tete (header)
        1 : affiche les informations de la tete (header)

appel : lireImage (image, 0 ou 1)
--------------------------------------------------------------------------- */
void lireImage (Image *image, int info)
{
  lireTete (image, info);

  if ((image->tabcoul =  (Uchar *) malloc (image->tete->rasMapLongueur))
                      == (Uchar *) NULL)
  {
      printf ("\nInfo : Erreur avec le 'MALLOC' de la table des couleurs.\n");
      exit (-5);

  } /* if then */

  if (fread (image->tabcoul, 1,
             image->tete->rasMapLongueur, image->fichiers.fde) !=
      image->tete->rasMapLongueur)
  {
     printf ("\nInfo : Impossible de lire la table des couleurs du",
                     " fichier-image '%s'.\n",
                      *image->fichiers.fichierS);
     exit (-6);

  } /* if then */

  if ((image->imgSV =  (Uchar *) malloc (image->tete->rasLongueur))
                    == (Uchar *) NULL)
  {
     printf ("\nInfo : Erreur avec le 'MALLOC' de l'image source.\n");
     exit (-5);

  } /* if then */

  if ((fread (image->imgSV, 1,
              image->tete->rasLongueur, image->fichiers.fde)) !=
       image->tete->rasLongueur)
  {
    printf("\nInfo : Impossible de lire les pixels du fichier-image",
                   " '%s'.\n", *image->fichiers.fichierS);
    exit (-7);

  } /* if then */

} /* lireImage */

/* ---------------------------------------------------------------------------
Fonction ECRIREIMAGE : Cette fonction ecrit les informations relatives a une
                       image (tete, table des couleurs et les pixels de
l'image).  Les pixels a ecrire DOIVENT ETRE dans la variable 'imgDV'.

appel : ecrireImage (image)
--------------------------------------------------------------------------- */
void ecrireImage (Image *image)
{
  ecrireTete(image);

  if (fwrite (image->tabcoul, 1,
              image->tete->rasMapLongueur, image->fichiers.fds) !=
      image->tete->rasMapLongueur)
  {
     printf ("\nInfo : Impossible d'crire la table des couleurs du",
                     " fichier-image '%s'.\n", image->fichiers.fichierD);
     exit (-9);

  } /* if then */

  if ((fwrite(image->imgDV, 1,
              image->tete->rasLongueur, image->fichiers.fds)) !=
       image->tete->rasLongueur)
  {
     printf ("\nInfo : Impossible d'ecrire les pixels au",
                     " fichier-image '%s'.\n", image->fichiers.fichierD);
     exit (-10);

  } /* if then */

} /* ecrireImage */

/* ---------------------------------------------------------------------------
Fonction ECRIREIMAGE_RAW : Cette fonction ecrit les informations relatives a une
                       image RAW (les pixels de
l'image).  Les pixels a ecrire DOIVENT ETRE dans la variable 'imgDV'.

appel : ecrireImage_RAW (image)
--------------------------------------------------------------------------- */
void ecrireImage_RAW (Image *image)
{

  if ((fwrite(image->imgDV, 1,
              image->tete->rasLongueur, image->fichiers.fds)) !=
       image->tete->rasLongueur)
  {
     printf ("\nInfo : Impossible d'ecrire les pixels au",
                     " fichier-image '%s'.\n", image->fichiers.fichierD);
     exit (-10);

  } /* if then */

} /* ecrireImage */


/* ---------------------------------------------------------------------------
Fonction ECRIREIMAGE_TABCOUMOD : 
Cette fonction ecrit les informations relatives a une image (tete, table des couleurs et les pixels de
l'image).  Les pixels a ecrire DOIVENT ETRE dans la variable 'imgDV'.

appel : ecrireImage_tabcoumod (image)
--------------------------------------------------------------------------- */
void ecrireImage_tabcoumod (Image *image)
{
  int i,j;
  Uchar tabcouRGB[(int)(3*(PIXMAX+1))];

  for(i=0;i<NBCANON;i++)
     for(j=0;j<=PIXMAX;j++)
        tabcouRGB[(int)(i*(PIXMAX+1)+j)] = j;

  image->tete->rasMapLongueur =  LONGUEUR_MAX_TABCOU;
  ecrireTete(image);

  if ((fwrite(tabcouRGB, 1,
       image->tete->rasMapLongueur, image->fichiers.fds)) !=
      image->tete->rasMapLongueur)
  {
     printf ("\nInfo : Impossible d'crire la table des couleurs du",
                     " fichier-image '%s'.\n", image->fichiers.fichierD);
     exit (-9);

  } /* if then */

  if ((fwrite(image->imgDV, 1,
              image->tete->rasLongueur, image->fichiers.fds)) !=
       image->tete->rasLongueur)
  {
     printf ("\nInfo : Impossible d'ecrire les pixels au",
                     " fichier-image '%s'.\n", image->fichiers.fichierD);
     exit (-10);

  } /* if then */

} /* ecrireImage */


/* ---------------------------------------------------------------------------
Fonction FERMERIMAGE : Cette fonction ferme les fichiers utilises pendant le
                       traitement et libere l'espace alloue par la structure
de l'image.

appel : fermerImage (&image)
--------------------------------------------------------------------------- */
void fermerImage (Image **image)
{
  Image *imageTmp;

  imageTmp = *image;

  if (imageTmp != (Image *) NULL)
  {
     fclose (imageTmp->fichiers.fde);
     fclose (imageTmp->fichiers.fds);

     if (imageTmp->tete != (TeteRaster *) NULL)
        free ((Uchar *) imageTmp->tete);

     if (imageTmp->imgSV != (Uchar *) NULL)
        free (imageTmp->imgSV);

     if (imageTmp->imgDV != (Uchar *) NULL)
        free (imageTmp->imgDV);

     if (imageTmp->tabcoul != (Uchar *) NULL)
        free (imageTmp->tabcoul);

     free ((Uchar *) imageTmp);

     *image = (Image *) NULL;

  } /* if then */

} /* fermerImage */

